# -*- coding: iso-8859-1 -*-
## ---------------------------------------------------------------------------
## PyCoinCoin - Module cookie explorer
##   Permet la rcupration des cookies
##   Author : renardjb@free.fr
##   Version : 1.0a  Fix et test sous linux
##   Version : 1.0   Initial version
## ---------------------------------------------------------------------------
import os
import os.path

class CookieExplorer:
    """ Rcupre les cookies dans les fichiers des navigateurs.
    """
    def __init__(self, server_name, filters=['login','md5','unique_id']):
        """ server_name : Nom du server web (ex: linuxfr.org)
        filters : Tableau contenant les cookies  rcuprer
        """
        self.site=server_name
        self.filters=filters

        self.ResetResult()
        # Gre la liste des navigateurs qui seront pris en charge
        self.__browser={}
        self.__browser['Firefox']=self.getFireTrucCookies
        if os.path.sep=="\\":
                self.__browser['Internet Explorer']=self.getIECookies
        self.__browser['Konqueror']=self.getKonquiCookies

    def ResetResult(self):
        """ Fait une remise  zro des recherches prcdentes.
        """
        self.result={} # Contiendra le rsultat de la recherche.        
        for filter in self.filters:
            self.result[filter]=""

    def __set_cookie(self, name, value):
        """ accepte la mise  jour uniquement si name est dj une
        valeur existante (MAJ chrisix: ou bien si filters est vide)
        """
        if not self.filters or self.result.has_key(name):
            self.result[name]=value

    def getCapacityBrowsersList(self):
        """ retourne la liste des navigateurs compatibles
        """
        return self.__browser.keys()

    def SearchCookies(self, browsername):
        """ Effectue la recherche des cookies.
        browsername = le nom du navigateur pris en charge.
        """
        self.__browser[browsername]()
        
    def getFirePath(self):
        """ Le path de Firefox est dans le rpertoire :
        sous windows xp, 2000:
        ~\\Application Data\\Mozilla\\Firefox\\*.default\\cookies.txt
        sous linux: ~/.mozilla/.firefox/*.default/cookies.txt
        sous win95,98:
        C:\\WINDOWS\\Application Data\\Mozilla\\Firefox\\Profiles\\xxx.default\\
        sous Mac OS X qui puxor
        ~/Library/Application Support/Firefox/Profiles/xxxxxxxx.default/

        retourne un tableau contenant les fichiers ou sont stocks les cookies.
        
        Test et approuv sur windows XP et 2000, et linux slackware (A FAIRE).
        """
        res=[]
        # on essaie de trouver un path valide (ouf :-()
        tmp_path=os.path.join(os.path.expanduser('~'),'Application Data',
                              'Mozilla','Firefox','Profiles')
        if os.path.exists(tmp_path)==False:
            # on tente la version unix
            tmp_path=os.path.join(os.path.expanduser('~'),'.mozilla','firefox')
            if os.path.exists(tmp_path)==False:
                # la version Mac Os X
                tmp_path=os.path.join(os.path.expanduser('~'),'Library',
                                       'Application Support',
                                       'Firefox', 'Profiles')
                if os.path.exists(tmp_path)==False:
                    # La version win9x, aprs c'est fini
                    tmp_path=os.path.join("C:",'WINDOWS','Application Data',
                                          'Mozilla','Firefox','Profiles')
                    if os.path.exists(tmp_path)==False:
                        print u"Ton OS puxor, on peut rien faire pour toi. !!!"
                        return res
        # maintenant, on liste les rpertoires qui contiennent uniquement
        # le fichier cookies.txt
        for path in os.listdir(tmp_path):
            tmp= os.path.join(tmp_path, path, 'cookies.txt')
            if os.path.exists(tmp)==True:
                res.append(tmp)
        return res

    def getKonquiCookies(self):
        filecookietxt=os.path.join(os.path.expanduser("~"),'.kde','share',
                                   'apps', 'kcookiejar','cookies')
        if os.path.exists(filecookietxt)==True:
            #il n'y a plus qu' analyser le fichier
            try:
                f=open(filecookietxt,'r')
                for line in f:
                    if self.site == line[:len(self.site)]:
                        cookies=line.split("\"")
			cookies=cookies[4].split(" ")
                        print cookies[-1] #cookies[6][:-1])
                        self.__set_cookie(cookies[15],cookies[-1][:-1])
            finally:
                f.close()

    def getFireTrucCookies(self):
        for filecookietxt in self.getFirePath():
            #il n'y a plus qu' analyser le fichier
            try:
                f=open(filecookietxt,'r')
                for line in f:
                    if self.site == line[:len(self.site)]:
                        cookies=line.split("\t")
                        self.__set_cookie(cookies[5],cookies[6][:-1])
            finally:
                f.close()

    def getIECookies(self):
        """ Le path d'IE est compos de
        ~\Cookies\username@site_sans_le_tld[Un_numro].txt
        C'est donc la merde, donc on va faire ce qu'on peut pour que ca marche.
        
        Test et approuv sur windows XP et 2000.
        """
        filename=os.path.join(os.path.expanduser("~"), "Cookies")
        filename=os.path.join(filename, os.getenv("USERNAME"))
        # on supprime le TLD
        site=self.site[0:self.site.rfind('.')]
        # et on recherche dans file[X].txt
        for index in xrange(5):
            tmp_filename=filename+"@" + site +"["+str(index)+"].txt"
            self.__parseIECookie(tmp_filename)
        return self.result
            
    def __parseIECookie(self, pathname):
        if os.path.exists(pathname)==True:
            try:
                f=open(pathname,'r')
                oldline=""
                for line in f:
                    for filter in self.filters:
                        if oldline==filter:
                            self.result[oldline]=line[:-1]
                    oldline = line [:-1]
            finally:
                f.close()
                
#------------------------------ Module de test --------------------------------
if __name__=='__main__':
    # --- test avec moules.org ---
    c=CookieExplorer('moules.org', filters=['PHPSESSID'])
    print "Liste des navigateurs pris en charge",c.getCapacityBrowsersList()
    print "-"*78
    for browser in c.getCapacityBrowsersList():
        print browser
        c.SearchCookies(browser)
        print c.result
        c.ResetResult()
    print "Liste des navigateurs pris en charge",c.getCapacityBrowsersList()
    
    # --- test avec DLFP ---
    c=CookieExplorer('linuxfr.org')
    for browser in c.getCapacityBrowsersList():
        print browser
        c.SearchCookies(browser)
        print c.result
        c.ResetResult()
    print "Liste des navigateurs pris en charge",c.getCapacityBrowsersList()
