# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module plugin
##   Ce module contient les classes gnriques d'interface des plug-ins
## ----------------------------------------------------------------------

import sys, os, os.path, urllib
sys.path.append('..')
import logging
log = logging.getLogger('pycc')

from constants import *
from utils import *
from persistants import Style, OptionSet, Option, Identity, Condition, ConditionSet, Filter, ActionPlugin


class Plugin(object):
  
  def __init__(self, pycc, name, descr="", categ=""):
    self.pycc = pycc    # Pointeur vers pycc
    self.name = name    # Le nom tel qu'il apparat dans le menu
    self.descr = descr  # La description longue
    self.categ = categ  # La catgorie, pour la hirarchisation en menu
    self.config_path = os.path.join(self.pycc.config_path, "plugins")
    self.style_name = 'plugin_%s' % self.name
    self.load_config()
    self.init()
    self.load_style()
  
  def __str__(self):
    return self.name
  
  def load_style(self):
    self.style = self.get_style()
    if self.style is not None:
      if not self.pycc.default_styles.exists(self.style_name):
        self.pycc.default_styles.add(self.style_name, self.style)
  
  def set_style_bouchot(self, bouchot):
    if self.style is not None:
      if not bouchot.styles.exists(self.style_name):
        bouchot.styles.add(self.style_name, self.pycc.default_styles.get(self.style_name))
        bouchot.styles.get(self.style_name).use_default = 1
        bouchot.apply_styles()
  
  def get_style(self):
    """ Renvoie un objet Style spcifique au plugin, s'il y a lieu """
    return None
  
  def load_config(self):
    """ Affecte  self.options l'ensemble des options de config du plug-in """
    self.options = None
  
  def init(self):
    """ Actions  raliser au moment de l'initialisation du plug-in """
    pass
  
  def config_frame(self, master):
    """ Construit la frame de configuration des options du plug-in """
    return None
  
  def process(self, bouchot, context, *args, **kwargs):
    """ Dfinit quoi faire lors de l'appel via une action """
    pass
  
  def action_menu(self):
    """ Dfinit quoi faire lors de l'appel via le menu """
    pass


class TextPlugin(Plugin):
  """ Classe spciale de plugins "transformeurs de texte"
  """

  def __init__(self, pycc, name, descr="", categ="TextTransform"):
    Plugin.__init__(self, pycc, name, descr, categ)
  
  def get_style(self):
    return None
    # return Style("[plugin] %s" % self.name, foreground='#ff0000')
  
  def action_menu(self):
    if self.pycc.palmi.palmi.selection_present():
      text = toUnicode(self.pycc.palmi.palmi.selection_get())
    else:
      ptxt = self.pycc.palmi.get()
      if len(ptxt) == 0: return
      text = ptxt[:-1]
    res = self.text_transform(text)
    self.pycc.palmi.palmi.inserter(res, del_selection=1)
  
  def text_transform(self, text, tags={}):
    return text
  
  def process(self, bouchot, context, post=None, *args, **kwargs):
    if context == PALMI:
      self.action_menu()
      return
    if post is None:
      return
    post.message = self.text_transform(post.message, post.tags)
    post.status.append(self.name)
    text, tags = bouchot.trib.formatPost(post)
    bouchot.trib.replaceFromTag("id%s" % post.id, text, tags)
    if self.style is not None:
      self.set_style_bouchot(bouchot)
      bouchot.trib.pinni.setStyle(post.id, self.style_name, 'message')
  

class AttribPlugin(Plugin):
  
  def __init__(self, pycc, name, descr="", categ="Attributs"):
    Plugin.__init__(self, pycc, name, descr, categ)
    # attlist = self.pycc.options.val('available_attrs')
    if self.attrib not in available_attrs:
      #attlist[self.attpos:self.attpos] = [self.attrib]
      #self.pycc.options.set('available_attrs', attlist)
      available_attrs[self.attpos:self.attpos] = [self.attrib]
  
  def init(self):
    self.attrib = 'attribname'
    self.attpos = 0  # Attribute position
  
  def load_style(self):
    self.style_name = self.attrib
    Plugin.load_style(self)
  
  def action_menu(self):
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded: return
    attlist = cur_trib.options.val('disp_post_attrs')
    if self.attrib in attlist:
      attlist.remove(self.attrib)
    else:
      attlist[self.attpos:self.attpos] = [self.attrib]
    cur_trib.options.set('disp_post_attrs', attlist)
    cur_trib.refresh_all()
  
  def compute_attrib(self, post):
    return "value"  # Abstract - calcul de la valeur du nouvel attribut de post
  
  def process(self, bouchot, context=PINNI, post=None, *args, **kwargs):
    if post is None: return
    post.__dict__[self.attrib] = self.compute_attrib(post)
    self.set_style_bouchot(bouchot)
  

class HttpBotOptions(OptionSet):
  
  def setObjects(self):
    self._objects = {  
      'bot_name':   Option("Nom du bot", value=""),
      'bot_server': Option("Serveur HTTP du bot", value=""),
      'bot_url':    Option("URL du bot", value=""),
      'bot_data':   Option("Donnes  envoyer au bot", value=""),
      'bot_id':     Option("Identit pour poster", value="", type=TYPE_CHOICE),
      'toedit': Option("Editables", {"Gnral": ['bot_name', 'bot_server', 'bot_url', 'bot_data', 'bot_id']})
    }

class BotOptions(OptionSet):
  
  def setObjects(self):
    self._objects = {  
      'bot_name':   Option("Nom du bot", value=""),
      'bot_id':     Option("Identit pour poster", value="", type=TYPE_CHOICE),
      'toedit': Option("Editables", {"Gnral": ['bot_name', 'bot_id']})
    }

from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor
from cointklib.dialogs import CoinDialog

class BotOptionsEditor(OptionsEditor):
  
  def disp_choice(self, fr, name, option):
    self.widgets[name] = ComboField(fr, option.descr, self.extra_arg.pycc.identities.get().keys(), value=option.get())
    self.widgets[name].pack(pady=2, fill=X)

class BotOptionsDialog(CoinDialog):
  
  def __init__(self, title, parent, object, extra_arg):
    self.object = object
    self.plugin = extra_arg
    CoinDialog.__init__(self, parent, title, ["Lancer", "Stopper", "Fermer"], modal=1)
  
  def body(self, master):
    self.w_options = BotOptionsEditor(master, self.object, extra_arg=self.plugin)
    self.w_options.pack()
  
  def bind_button(self, button):
    if button == 'Stopper':
      return self.closeStop
    elif button == 'Lancer':
      return self.closeStart
    else:
      return self.cancel
  
  def closeStop(self):
    self.apply()
    self.plugin.stop()
    self.cancel()
  
  def closeStart(self):
    self.apply()
    self.plugin.start()
    self.cancel()
  
  def apply(self):
    self.w_options.apply()

class BotPlugin(Plugin):

  def __init__(self, pycc, name, descr="", categ="Bots"):
    self.started = 0
    Plugin.__init__(self, pycc, name, descr, categ)
  
  def load_config(self):
    self.options = BotOptions(name=self.name, path=self.config_path)
    self.set_defaults()
    self.options.load()
  
  def set_defaults(self):
    pass # Methode abstraite
  
  def config_frame(self, master):
    return BotOptionsEditor(master, self.options, self.pycc)
  
  def action_menu(self):
    BotOptionsDialog(self.name, self.pycc.root, self.options, self)
  
  def start(self, data=None):
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded:
      return
    myfilter, pos = self.find_filter()
    if myfilter is None:
      tribcond = Condition('trib', OP_MATCHES, '(%s)' % cur_trib.getName())
      regexp = "".join(["[%s%s]" % (c.upper(), c.lower()) for c in self.options.val('bot_name')]) + "[<]"
      botcond = Condition('message', OP_MATCHES, regexp)
      condset  = ConditionSet([tribcond, botcond], COND_ALL)
      action = ActionPlugin(self.name)
      newfilter = Filter("Bot_%s" % self.options.val('bot_name'), [action], condset, 1, 1)
      self.pycc.processor.add_filter(newfilter)
    else:
      triblist = myfilter.conditions.conditions[0].expr[1:-1].split("|")
      if cur_trib.getName() not in triblist:
        triblist.append(cur_trib.getName())
        myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
    if not self.started:
      self.start_process(data)
    self.started = 1
  
  def start_process(self, data=None):
    pass # Abstraite
  
  def stop(self):
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded:
      return
    myfilter, pos = self.find_filter()
    if myfilter is not None:
      triblist = [trib for trib in myfilter.conditions.conditions[0].expr[1:-1].split("|") if trib != cur_trib.getName()]
      if len(triblist) > 0:
        myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
      else:
        self.pycc.processor.remove_filter(pos)
    self.started = 0
  
  def find_filter(self):
    myfilter = None
    pos = 0
    for filter in self.pycc.processor.get():
      if filter.name == "Bot_%s" % self.options.val('bot_name'):
        myfilter = filter
        break
      pos += 1
    return myfilter, pos
  
  def bot_process(self, text):
    return "" # Mthode abstraite
  
  def process(self, bouchot, context, post=None, *args, **kwargs):
    if context == PALMI:
      self.action_menu()
      return
    if not self.started or post is None:
      return
    text = post.message
    res = self.bot_process(text)
    if not res:
      self.error = "Bot %s: Rsultat vide" % self.name
      log.warning(self.error)
      return
    topost = toUnicode(post.dispTimeRef()+" ") + toUnicode(res)
    curid = self.options.val('bot_id')
    if not curid:
      self.error = "Bot %s: Impossible de poster, identit non dfinie" % self.name
      log.error(self.error)
      return
    id = self.pycc.identities.get()[curid]
    id_clone = Identity()
    id_clone.cookies = id.cookies
    id_clone.login = id.login
    id_clone.ua = self.name
    try:
      bouchot.trib.poster(topost, 1, id_clone)
    except:
      self.error = "Bot %s: Impossible de poster dans le bouchot %s" % (self.name, bouchot.name)
      log.error(self.error)

  
class HttpBotPlugin(BotPlugin):
  
  def __init__(self, pycc, name, descr="", categ="Bots"):
    self.cookie = None
    BotPlugin.__init__(self, pycc, name, descr, categ)
  
  def load_config(self):
    self.options = HttpBotOptions(name=self.name, path=self.config_path)
    self.set_defaults()
    self.options.load()
  
  def start_process(self, data=None):
    self.getcookie(data)
  
  def getcookie(self, data=None):
    server = self.options.val('bot_server')
    res = ""
    try:
      f, cookies = open_url_with_cookies(server, self.options.val('bot_url'), self.pycc.options, data=data)
      res = f.read()
      f.close()
      self.cookie = ";".join(["%s=%s" % (cookie.name, cookie.value) for cookie in cookies if cookie.domain in server])
    except:
      self.error = "Bot %s: Impossible de recuperer le cookie de session" % self.name
      log.warning(self.error)
    print "## Mes cookies=", self.cookie
    return res
  
  def bot_query(self, text, data={}, method='post'):
    tname = self.options.val('bot_data')
    try:
      text = text.encode('utf-8')
    except:
      log.warning("Bot %s: texte d'entre pas utf-8isable" % self.name)
    data[tname] = text
    if self.cookie:
      for cookie in self.cookie.split(";"):
        c_name, c_val = cookie.split('=')
        data[c_name] = c_val
    query = urllib.urlencode(data)
    server = self.options.val('bot_server')
    url = self.options.val('bot_url')
    print "### Bot Plugin: query=", query
    try:
      if method == 'post':
        f = open_url(server, url, self.pycc.options, cookie=self.cookie, data=query)
      else:
        url += "?"+query
        f = open_url(server, url, self.pycc.options, cookie=self.cookie)
      res = f.read()
      f.close()
    except:
      self.error = "Bot %s: Erreur rseau ou du serveur" % self.name
      log.error(self.error)
      res = None
    return res
  
  def parse_res(self, html):
    # Mthode abstraite
    return ""
  
  def bot_process(self, texte):
    try:
      text = texte.encode(bouchot.trib.options.val('encoding'))
    except:
      text = texte
    html_res = self.bot_query(text[text.find("<")+1:])
    if not html_res:
      self.error = "Bot %s: Rsultat de requte vers le bot vide" % self.name
      log.warning(self.error)
      return
    res = self.parse_res(html_res)
    return res
  